'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _Transformer = require('./Transformer');

var _Transformer2 = _interopRequireDefault(_Transformer);

var _class = require('./transform/class');

var _class2 = _interopRequireDefault(_class);

var _template = require('./transform/template');

var _template2 = _interopRequireDefault(_template);

var _arrow = require('./transform/arrow');

var _arrow2 = _interopRequireDefault(_arrow);

var _let = require('./transform/let');

var _let2 = _interopRequireDefault(_let);

var _defaultParam = require('./transform/defaultParam');

var _defaultParam2 = _interopRequireDefault(_defaultParam);

var _destructParam = require('./transform/destructParam');

var _destructParam2 = _interopRequireDefault(_destructParam);

var _argSpread = require('./transform/argSpread');

var _argSpread2 = _interopRequireDefault(_argSpread);

var _argRest = require('./transform/argRest');

var _argRest2 = _interopRequireDefault(_argRest);

var _objMethod = require('./transform/objMethod');

var _objMethod2 = _interopRequireDefault(_objMethod);

var _objShorthand = require('./transform/objShorthand');

var _objShorthand2 = _interopRequireDefault(_objShorthand);

var _noStrict = require('./transform/noStrict');

var _noStrict2 = _interopRequireDefault(_noStrict);

var _commonjs = require('./transform/commonjs');

var _commonjs2 = _interopRequireDefault(_commonjs);

var _exponent = require('./transform/exponent');

var _exponent2 = _interopRequireDefault(_exponent);

var _multiVar = require('./transform/multiVar');

var _multiVar2 = _interopRequireDefault(_multiVar);

var _forOf = require('./transform/forOf');

var _forOf2 = _interopRequireDefault(_forOf);

var _forEach = require('./transform/forEach');

var _forEach2 = _interopRequireDefault(_forEach);

var _includes = require('./transform/includes');

var _includes2 = _interopRequireDefault(_includes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var transformsMap = {
  'class': _class2.default,
  'template': _template2.default,
  'arrow': _arrow2.default,
  'let': _let2.default,
  'default-param': _defaultParam2.default,
  'destruct-param': _destructParam2.default,
  'arg-spread': _argSpread2.default,
  'arg-rest': _argRest2.default,
  'obj-method': _objMethod2.default,
  'obj-shorthand': _objShorthand2.default,
  'no-strict': _noStrict2.default,
  'commonjs': _commonjs2.default,
  'exponent': _exponent2.default,
  'multi-var': _multiVar2.default,
  'for-of': _forOf2.default,
  'for-each': _forEach2.default,
  'includes': _includes2.default
};

/**
 * Central place for accessing all the builtin transforms
 */
exports.default = {
  /**
   * Factori method for creating a Transformer
   * by just specifying the names of the transforms.
   * @param  {String[]} transformNames
   * @return {Transformer}
   */
  createTransformer: function createTransformer(transformNames) {
    var _this = this;

    return new _Transformer2.default(transformNames.map(function (name) {
      return _this.get(name);
    }));
  },


  /**
   * Maps transform name to the actual transform function
   * @param  {String} name
   * @return {Function} the transform or undefined when no such transform
   */
  get: function get(name) {
    return transformsMap[name];
  }
};
module.exports = exports['default'];